require "benchmark"
require "cocaine"

module Dumptruck
  class Truck

    #mysqldump needs to be installed locally, you can pass in the location if Ruby doesn't find it
    attr_accessor :mysqldump_bin

    #host, hostname where you are connecting too, defaults too localhost
    attr_accessor :host

    #port, other than 3306
    attr_accessor :port

    #filename, default is `output`, the filename will have a time/date string appended to it.
    attr_accessor :filename

    #username for the host
    attr_accessor :username

    #password for host
    attr_accessor :password

    #output_dir is where you'd like the output file to live after its run
    attr_accessor :output_dir

    def initialize(params)
      defaults = {
        output_dir: "/tmp",
        filename: "output",
        port: 3306,
        host: "localhost"
      }.merge!(params)

      @username      = defaults[:username]
      @password      = defaults[:password]
      @host          = defaults[:host]
      @port          = defaults[:port]
      @filename      = defaults[:filename]
      @output_dir    = defaults[:output_dir]

      Cocaine::CommandLine.path = params[:mysqldump_bin]
    end

    def credentials
      creds = "-u #{@username} "
      creds += "--password=#{@password} " unless @password.nil?
      creds += "-h #{@host} "
      creds += "-P #{@port}" unless @port.nil?
      creds
    end

    def output
      "#{@output_dir}/#{@filename}_#{Time.now.strftime("%Y-%m-%d-%H%M")}.sql.gz"
    end

    def load( input )
      @loads = [] unless @loads
      if input.instance_of? Dumptruck::Load #created separate from the truck
        @loads << input
      else
        input = Dumptruck::Load.new(input)
        @loads << input
      end

      input
    end

    def loads
      @loads.map
    end

    def gzip
      "| gzip -c >>"
    end

    def command
      self.loads.each_with_index do |l,i|
        mysqldump = Cocaine::CommandLine.new('mysqldump',
          "#{self.credentials} #{l.options} #{l.ignored_tables}#{l.database}"
        )
        mysqldump.command
      end
    end

    def dump
      raise Dumptruck::NothingLoaded, "You haven't loaded anything!" unless @loads
      filename = self.output

      time = Benchmark.realtime do
        self.loads.each_with_index do |l,i|
          cmd = Cocaine::CommandLine.new("mysqldump",
            "#{self.credentials} #{l.options} #{l.ignored_tables}#{l.database} #{self.gzip} #{filename}"
          )

          begin
              cmd.run
          rescue Cocaine::ExitStatusError => e
            e
          rescue Cocaine::CommandNotFoundError => e
            raise Dumptruck::MysqlDumpNotFound, "Could not find the `mysqldump` command."
          end
        end
      end
         file = File.open(filename)
        {time: time, file: file}
    end
  end
end

